@@ -313,6 +313,9 @@ class TestContentRepository(unittest.TestCase):
         self.content_repository.has_torrent = lambda infohash: False
         self.content_repository.get_torrent = lambda infohash: None
 
+        self.content_repository.torrent_db._db = MockObject()
+        self.content_repository.torrent_db._db.commit_now = lambda x=None: None
+
         self.content_repository.called_update_torrent = False
         self.content_repository.update_from_torrent_search_results(search_results.values())
         self.assertTrue(self.content_repository.called_update_torrent)
@@ -342,7 +345,6 @@ class TestContentRepositoryWithRealDatabase(TestBase):
         tar.extractall(session_base_dir)
         db_path = os.path.join(session_base_dir, 'bak_new_tribler.sdb')
         self.sqlitedb = SQLiteCacheDB(db_path, busytimeout=BUSYTIMEOUT)
-        self.sqlitedb.initialize()
 
         session = MockObject()
         session.sqlite_db = self.sqlitedb
@@ -355,7 +357,7 @@ class TestContentRepositoryWithRealDatabase(TestBase):
     def tearDown(self):
         self.torrent_db.close()
         self.sqlitedb.close()
-        super(TestContentRepositoryWithRealDatabase, self).tearDown()
+        return super(TestContentRepositoryWithRealDatabase, self).tearDown()
 
     def test_update_db_from_search_results(self):
         """
