@@ -18,6 +18,8 @@ class TestTriblerConfig(TriblerCoreTest):
         super(TestTriblerConfig, self).setUp()
 
         self.tribler_config = TriblerConfig()
+        self.tribler_config.get_default_state_dir = lambda **_: self.getStateDir()
+        self.tribler_config.set_state_dir(self.getStateDir())
         self.assertIsNotNone(self.tribler_config)
 
     def test_init_with_config(self):
@@ -45,7 +47,7 @@ class TestTriblerConfig(TriblerCoreTest):
         port = 4444
         self.tribler_config.set_anon_listen_port(port)
         self.tribler_config.write()
-        path = os.path.join(self.tribler_config.get_state_dir(), FILENAME)
+        path = os.path.join(self.tribler_config.get_default_state_dir(), FILENAME)
         read_config = TriblerConfig.load(path)
 
         read_config.validate()
@@ -103,8 +105,6 @@ class TestTriblerConfig(TriblerCoreTest):
         self.tribler_config.set_family_filter_enabled(False)
         self.assertEqual(self.tribler_config.get_family_filter_enabled(), False)
 
-        self.tribler_config.set_state_dir(None)
-        self.assertEqual(self.tribler_config.get_state_dir(), self.tribler_config.get_default_state_dir())
         self.tribler_config.set_state_dir("TEST")
         self.assertEqual(self.tribler_config.get_state_dir(), "TEST")
 
@@ -159,6 +159,8 @@ class TestTriblerConfig(TriblerCoreTest):
         self.assertEqual(self.tribler_config.get_http_api_enabled(), True)
         self.tribler_config.set_http_api_port(True)
         self.assertEqual(self.tribler_config.get_http_api_port(), True)
+        self.tribler_config.set_http_api_retry_port(True)
+        self.assertTrue(self.tribler_config.get_http_api_retry_port())
 
     def test_get_set_methods_dispersy(self):
         """
@@ -248,6 +250,10 @@ class TestTriblerConfig(TriblerCoreTest):
         self.assertEqual(self.tribler_config.get_default_safeseeding_enabled(), True)
         self.tribler_config.set_default_destination_dir(True)
         self.assertEqual(self.tribler_config.get_default_destination_dir(), True)
+        self.tribler_config.set_tunnel_community_random_slots(10)
+        self.assertEqual(self.tribler_config.get_tunnel_community_random_slots(), 10)
+        self.tribler_config.set_tunnel_community_competing_slots(20)
+        self.assertEqual(self.tribler_config.get_tunnel_community_competing_slots(), 20)
 
     def test_get_set_methods_torrent_store(self):
         """
@@ -373,3 +379,10 @@ class TestTriblerConfig(TriblerCoreTest):
         """
         self.tribler_config.set_dht_enabled(False)
         self.assertFalse(self.tribler_config.get_dht_enabled())
+
+    def test_get_set_methods_record_transactions(self):
+        """
+        Check whether record_transactions get and set methods are working as expected.
+        """
+        self.tribler_config.set_record_transactions(True)
+        self.assertTrue(self.tribler_config.get_record_transactions())
