@@ -1,34 +1,36 @@
+from __future__ import absolute_import
+
+import datetime
+import logging
 import os
 import socket
 from time import localtime, strftime
 
-import datetime
-import matplotlib
-from PyQt5.QtCore import QTimer
-from PyQt5.QtCore import pyqtSignal
+from PyQt5 import QtGui, uic
+from PyQt5.QtCore import QTimer, Qt, pyqtSignal
 from PyQt5.QtGui import QTextCursor
-from PyQt5.QtWidgets import QFileDialog, QTextEdit, QDesktopWidget, QTreeWidget
-from PyQt5.QtWidgets import QMessageBox
-from PyQt5.QtWidgets import QSizePolicy
-from TriblerGUI.dialogs.confirmationdialog import ConfirmationDialog
-from meliae import scanner
-from twisted.internet.task import LoopingCall
+from PyQt5.QtWidgets import QDesktopWidget, QFileDialog, QHeaderView, QMainWindow, QMessageBox, QSizePolicy, \
+    QTreeWidgetItem
 
+import matplotlib
 matplotlib.use('Qt5Agg')
-
-import psutil
-from PyQt5 import uic, QtGui
-from PyQt5.QtCore import Qt
-from PyQt5.QtWidgets import QHeaderView
-from PyQt5.QtWidgets import QMainWindow, QTreeWidgetItem
 from matplotlib.backends.backend_qt5agg import FigureCanvas
 from matplotlib.dates import DateFormatter
 from matplotlib.figure import Figure
 
+from meliae import scanner
+
+import psutil
+
+from six.moves import xrange
+
 import Tribler.Core.Utilities.json_util as json
-from TriblerGUI.utilities import get_ui_file_path, format_size
-from TriblerGUI.tribler_request_manager import performed_requests as tribler_performed_requests, TriblerRequestManager
+
+from TriblerGUI.defs import DEBUG_PANE_REFRESH_TIMEOUT
+from TriblerGUI.dialogs.confirmationdialog import ConfirmationDialog
 from TriblerGUI.event_request_manager import received_events as tribler_received_events
+from TriblerGUI.tribler_request_manager import TriblerRequestManager, performed_requests as tribler_performed_requests
+from TriblerGUI.utilities import format_size, get_ui_file_path
 
 
 class MplCanvas(FigureCanvas):
@@ -112,6 +114,7 @@ class DebugWindow(QMainWindow):
     resize_event = pyqtSignal()
 
     def __init__(self, settings, tribler_version):
+        self._logger = logging.getLogger(self.__class__.__name__)
         QMainWindow.__init__(self)
 
         self.request_mgr = None
@@ -158,6 +161,9 @@ class DebugWindow(QMainWindow):
         # IPv8 statistics enabled?
         self.ipv8_statistics_enabled = settings['ipv8']['statistics']
 
+        # Libtorrent tab
+        self.init_libtorrent_tab()
+
         # Position to center
         frame_geometry = self.frameGeometry()
         screen = QDesktopWidget().screenNumber(QDesktopWidget().cursor().pos())
@@ -165,41 +171,78 @@ class DebugWindow(QMainWindow):
         frame_geometry.moveCenter(center_point)
         self.move(frame_geometry.topLeft())
 
+        # Refresh timer
+        self.refresh_timer = None
+
+    def hideEvent(self, hide_event):
+        self.stop_timer()
+
+    def run_with_timer(self, call_fn, timeout=DEBUG_PANE_REFRESH_TIMEOUT):
+        call_fn()
+        self.stop_timer()
+        self.refresh_timer = QTimer()
+        self.refresh_timer.setSingleShot(True)
+        self.refresh_timer.timeout.connect(lambda _call_fn=call_fn, _timeout=timeout:
+                                           self.run_with_timer(_call_fn, timeout=_timeout))
+        self.refresh_timer.start(timeout)
+
+    def stop_timer(self):
+        if self.refresh_timer:
+            try:
+                self.refresh_timer.stop()
+                self.refresh_timer.deleteLater()
+            except RuntimeError:
+                self._logger.error("Failed to stop refresh timer in Debug pane")
+
+    def init_libtorrent_tab(self):
+        self.window().libtorrent_tab_widget.setCurrentIndex(0)
+        self.window().libtorrent_tab_widget.currentChanged.connect(lambda _: self.load_libtorrent_data(export=False))
+
+        self.window().lt_zero_hop_btn.clicked.connect(lambda _: self.load_libtorrent_data(export=False))
+        self.window().lt_one_hop_btn.clicked.connect(lambda _: self.load_libtorrent_data(export=False))
+        self.window().lt_two_hop_btn.clicked.connect(lambda _: self.load_libtorrent_data(export=False))
+        self.window().lt_three_hop_btn.clicked.connect(lambda _: self.load_libtorrent_data(export=False))
+        self.window().lt_export_btn.clicked.connect(lambda _: self.load_libtorrent_data(export=True))
+
+        self.window().lt_zero_hop_btn.setChecked(True)
+
     def tab_changed(self, index):
         if index == 0:
             self.load_general_tab()
         elif index == 1:
             self.load_requests_tab()
         elif index == 2:
-            self.load_trustchain_tab()
+            self.run_with_timer(self.load_trustchain_tab)
         elif index == 3:
             self.ipv8_tab_changed(self.window().ipv8_tab_widget.currentIndex())
         elif index == 4:
             self.tunnel_tab_changed(self.window().tunnel_tab_widget.currentIndex())
         elif index == 5:
-            self.load_dht_tab()
+            self.run_with_timer(self.load_dht_tab)
         elif index == 6:
-            self.load_events_tab()
+            self.run_with_timer(self.load_events_tab)
         elif index == 7:
             self.system_tab_changed(self.window().system_tab_widget.currentIndex())
         elif index == 8:
+            self.load_libtorrent_data()
+        elif index == 9:
             self.load_logs_tab()
 
     def ipv8_tab_changed(self, index):
         if index == 0:
-            self.load_ipv8_general_tab()
+            self.run_with_timer(self.load_ipv8_general_tab)
         elif index == 1:
-            self.load_ipv8_communities_tab()
+            self.run_with_timer(self.load_ipv8_communities_tab)
         elif index == 2:
-            self.load_ipv8_community_details_tab()
+            self.run_with_timer(self.load_ipv8_community_details_tab)
 
     def tunnel_tab_changed(self, index):
         if index == 0:
-            self.load_tunnel_circuits_tab()
+            self.run_with_timer(self.load_tunnel_circuits_tab)
         elif index == 1:
-            self.load_tunnel_relays_tab()
+            self.run_with_timer(self.load_tunnel_relays_tab)
         elif index == 2:
-            self.load_tunnel_exits_tab()
+            self.run_with_timer(self.load_tunnel_exits_tab)
 
     def system_tab_changed(self, index):
         if index == 0:
@@ -673,3 +716,59 @@ class DebugWindow(QMainWindow):
         # and restore window with keeping maximized/normal state
         self.window().setWindowState(self.window().windowState() & ~Qt.WindowMinimized | Qt.WindowActive)
         self.window().activateWindow()
+
+    def load_libtorrent_data(self, export=False):
+        tab = self.window().libtorrent_tab_widget.currentIndex()
+        hop = 0 if self.window().lt_zero_hop_btn.isChecked() \
+            else 1 if self.window().lt_one_hop_btn.isChecked() \
+            else 2 if self.window().lt_two_hop_btn.isChecked() \
+            else 3
+        if tab == 0:
+            self.load_libtorrent_settings_tab(hop, export=export)
+        elif tab == 1:
+            self.load_libtorrent_sessions_tab(hop, export=export)
+
+    def load_libtorrent_settings_tab(self, hop, export=False):
+        self.request_mgr = TriblerRequestManager()
+        self.request_mgr.perform_request("libtorrent/settings?hop=%d" % hop,
+                                         lambda data: self.on_libtorrent_settings_received(data, export=export))
+        self.window().libtorrent_settings_tree_widget.clear()
+
+    def on_libtorrent_settings_received(self, data, export=False):
+        if not data:
+            return
+        for key, value in data["settings"].iteritems():
+            item = QTreeWidgetItem(self.window().libtorrent_settings_tree_widget)
+            item.setText(0, key)
+            item.setText(1, str(value))
+            self.window().libtorrent_settings_tree_widget.addTopLevelItem(item)
+        if export:
+            self.save_to_file("libtorrent_settings.json", data)
+
+    def load_libtorrent_sessions_tab(self, hop, export=False):
+        self.request_mgr = TriblerRequestManager()
+        self.request_mgr.perform_request("libtorrent/session?hop=%d" % hop,
+                                         lambda data: self.on_libtorrent_session_received(data, export=export))
+        self.window().libtorrent_session_tree_widget.clear()
+
+    def on_libtorrent_session_received(self, data, export=False):
+        if not data:
+            return
+        for key, value in data["session"].iteritems():
+            item = QTreeWidgetItem(self.window().libtorrent_session_tree_widget)
+            item.setText(0, key)
+            item.setText(1, str(value))
+            self.window().libtorrent_session_tree_widget.addTopLevelItem(item)
+        if export:
+            self.save_to_file("libtorrent_session.json", data)
+
+    def save_to_file(self, filename, data):
+        base_dir = QFileDialog.getExistingDirectory(self, "Select an export directory", "", QFileDialog.ShowDirsOnly)
+        if len(base_dir) > 0:
+            dest_path = os.path.join(base_dir, filename)
+            try:
+                torrent_file = open(dest_path, "wb")
+                torrent_file.write(json.dumps(data))
+                torrent_file.close()
+            except IOError as exc:
+                ConfirmationDialog.show_error(self.window(), "Error exporting file", str(exc))
