@@ -14,8 +14,7 @@ from threading import Event, enumerate as enumerate_threads
 from traceback import print_exc
 
 from Tribler.Core.CacheDB.sqlitecachedb import forceDBThread
-from Tribler.Core.DecentralizedTracking.dht_provider import MainlineDHTProvider
-from Tribler.Core.DownloadConfig import DownloadStartupConfig, DefaultDownloadStartupConfig
+from Tribler.Core.DownloadConfig import DefaultDownloadStartupConfig, DownloadStartupConfig
 from Tribler.Core.Modules.payout_manager import PayoutManager
 from Tribler.Core.Modules.resource_monitor import ResourceMonitor
 from Tribler.Core.Modules.search_manager import SearchManager
@@ -28,17 +27,17 @@ from Tribler.Core.TorrentDef import TorrentDef, TorrentDefNoMetainfo
 from Tribler.Core.Utilities.configparser import CallbackConfigParser
 from Tribler.Core.Utilities.install_dir import get_lib_path
 from Tribler.Core.Video.VideoServer import VideoServer
-from Tribler.Core.simpledefs import (NTFY_DISPERSY, NTFY_STARTED, NTFY_TORRENTS, NTFY_UPDATE, NTFY_TRIBLER,
-                                     NTFY_FINISHED, DLSTATUS_DOWNLOADING, DLSTATUS_STOPPED_ON_ERROR, NTFY_ERROR,
-                                     DLSTATUS_SEEDING, NTFY_TORRENT, STATE_STARTING_DISPERSY, STATE_LOADING_COMMUNITIES,
-                                     STATE_INITIALIZE_CHANNEL_MGR, STATE_START_MAINLINE_DHT, STATE_START_LIBTORRENT,
-                                     STATE_START_TORRENT_CHECKER, STATE_START_REMOTE_TORRENT_HANDLER,
-                                     STATE_START_API_ENDPOINTS, STATE_START_WATCH_FOLDER, STATE_START_CREDIT_MINING)
+from Tribler.Core.simpledefs import (DLSTATUS_DOWNLOADING, DLSTATUS_SEEDING, DLSTATUS_STOPPED_ON_ERROR, NTFY_DISPERSY,
+                                     NTFY_ERROR, NTFY_FINISHED, NTFY_STARTED, NTFY_TORRENT, NTFY_TORRENTS, NTFY_TRIBLER,
+                                     NTFY_UPDATE, STATE_INITIALIZE_CHANNEL_MGR, STATE_LOADING_COMMUNITIES,
+                                     STATE_STARTING_DISPERSY, STATE_START_API_ENDPOINTS, STATE_START_CREDIT_MINING,
+                                     STATE_START_LIBTORRENT, STATE_START_REMOTE_TORRENT_HANDLER,
+                                     STATE_START_TORRENT_CHECKER, STATE_START_WATCH_FOLDER)
 from Tribler.pyipv8.ipv8.dht.provider import DHTCommunityProvider
 from Tribler.pyipv8.ipv8.keyvault.private.m2crypto import M2CryptoSK
 from Tribler.pyipv8.ipv8.peer import Peer
 from Tribler.pyipv8.ipv8.peerdiscovery.churn import RandomChurn
-from Tribler.pyipv8.ipv8.peerdiscovery.deprecated.discovery import DiscoveryCommunity
+from Tribler.pyipv8.ipv8.peerdiscovery.community import DiscoveryCommunity, PeriodicSimilarity
 from Tribler.pyipv8.ipv8.peerdiscovery.discovery import EdgeWalk, RandomWalk
 from Tribler.pyipv8.ipv8.taskmanager import TaskManager
 from Tribler.pyipv8.ipv8.util import blockingCallFromThread, blocking_call_on_reactor_thread
@@ -50,6 +49,22 @@ from twisted.internet.threads import deferToThread
 from twisted.python.threadable import isInIOThread
 
 
+class DualStackDiscoveryCommunity(DiscoveryCommunity):
+    """
+    This is a stopgap measure until Dispersy is removed.
+    The reason for this class is that Dispersy bypasses IPv8's load balancing.
+    By injecting peers into IPv8, Dispersy then causes a peer explosion.
+    This subclass can be removed once Dispersy is gone.
+    """
+
+    def on_introduction_response(self, source_address, data):
+        if self.max_peers >= 0 and len(self.get_peers()) > self.max_peers:
+            self.logger.info("Dropping introduction response from (%s, %d): too many peers!",
+                             source_address[0], source_address[1])
+            return
+        return super(DualStackDiscoveryCommunity, self).on_introduction_response(source_address, data)
+
+
 class TriblerLaunchMany(TaskManager):
 
     def __init__(self):
@@ -99,7 +114,6 @@ class TriblerLaunchMany(TaskManager):
 
         self.video_server = None
 
-        self.mainline_dht = None
         self.ltmgr = None
         self.tracker_manager = None
         self.torrent_checker = None
@@ -141,7 +155,7 @@ class TriblerLaunchMany(TaskManager):
                     raise RuntimeError("Metadata store (leveldb) is None which should not normally happen")
 
             # torrent collecting: RemoteTorrentHandler
-            if self.session.config.get_torrent_collecting_enabled():
+            if self.session.config.get_torrent_collecting_enabled() and self.session.config.get_dispersy_enabled():
                 from Tribler.Core.RemoteTorrentHandler import RemoteTorrentHandler
                 self.rtorrent_handler = RemoteTorrentHandler(self.session)
 
@@ -187,7 +201,7 @@ class TriblerLaunchMany(TaskManager):
                 ipv8_config['keys'] = []  # We load the keys ourselves
 
                 if self.session.config.get_ipv8_bootstrap_override():
-                    import Tribler.pyipv8.ipv8.deprecated.community as community_file
+                    import Tribler.pyipv8.ipv8.community as community_file
                     community_file._DEFAULT_ADDRESSES = [self.session.config.get_ipv8_bootstrap_override()]
                     community_file._DNS_ADDRESSES = []
 
@@ -241,10 +255,11 @@ class TriblerLaunchMany(TaskManager):
             content = key_file.read()
         content = content[31:-30].replace('\n', '').decode("BASE64")
         peer = Peer(M2CryptoSK(keystring=content))
-        discovery_community = DiscoveryCommunity(peer, self.ipv8.endpoint, self.ipv8.network)
+        discovery_community = DualStackDiscoveryCommunity(peer, self.ipv8.endpoint, self.ipv8.network)
         discovery_community.resolve_dns_bootstrap_addresses()
         self.ipv8.overlays.append(discovery_community)
         self.ipv8.strategies.append((RandomChurn(discovery_community), -1))
+        self.ipv8.strategies.append((PeriodicSimilarity(discovery_community), -1))
 
         if not self.session.config.get_dispersy_enabled():
             self.ipv8.strategies.append((RandomWalk(discovery_community), 20))
@@ -279,20 +294,20 @@ class TriblerLaunchMany(TaskManager):
 
         # Tunnel Community
         if self.session.config.get_tunnel_community_enabled():
-
             from Tribler.community.triblertunnel.community import TriblerTunnelCommunity, TriblerTunnelTestnetCommunity
             community_cls = TriblerTunnelTestnetCommunity if self.session.config.get_testnet() else \
                 TriblerTunnelCommunity
 
-            if self.mainline_dht:
-                dht_provider = MainlineDHTProvider(self.mainline_dht, self.session.config.get_dispersy_port())
-            else:
-                dht_provider = DHTCommunityProvider(self.dht_community, self.session.config.get_dispersy_port())
+            random_slots = self.session.config.get_tunnel_community_random_slots()
+            competing_slots = self.session.config.get_tunnel_community_competing_slots()
 
+            dht_provider = DHTCommunityProvider(self.dht_community, self.session.config.get_dispersy_port())
             self.tunnel_community = community_cls(peer, self.ipv8.endpoint, self.ipv8.network,
                                                   tribler_session=self.session,
                                                   dht_provider=dht_provider,
-                                                  bandwidth_wallet=self.wallets["MB"])
+                                                  bandwidth_wallet=self.wallets["MB"],
+                                                  random_slots=random_slots,
+                                                  competing_slots=competing_slots)
             self.ipv8.overlays.append(self.tunnel_community)
             self.ipv8.strategies.append((RandomWalk(self.tunnel_community), 20))
 
@@ -306,7 +321,8 @@ class TriblerLaunchMany(TaskManager):
                                                   trustchain=self.trustchain_community,
                                                   dht=self.dht_community,
                                                   wallets=self.wallets,
-                                                  working_directory=self.session.config.get_state_dir())
+                                                  working_directory=self.session.config.get_state_dir(),
+                                                  record_transactions=self.session.config.get_record_transactions())
 
             self.ipv8.overlays.append(self.market_community)
 
@@ -398,14 +414,6 @@ class TriblerLaunchMany(TaskManager):
 
             self.session.readable_status = STATE_LOADING_COMMUNITIES
 
-        # We should load the mainline DHT before loading the IPv8 overlays since the DHT is used for the tunnel overlay.
-        if self.session.config.get_mainline_dht_enabled():
-            self.session.readable_status = STATE_START_MAINLINE_DHT
-            from Tribler.Core.DecentralizedTracking import mainlineDHT
-            self.mainline_dht = mainlineDHT.init(('127.0.0.1', self.session.config.get_mainline_dht_port()),
-                                                 self.session.config.get_state_dir())
-            self.upnp_ports.append((self.session.config.get_mainline_dht_port(), 'UDP'))
-
         # Wallets
         if self.session.config.get_bitcoinlib_enabled():
             try:
@@ -539,6 +547,7 @@ class TriblerLaunchMany(TaskManager):
             if isinstance(tdef, TorrentDefNoMetainfo):
                 self.torrent_db.addOrGetTorrentID(tdef.get_infohash())
                 self.torrent_db.updateTorrent(tdef.get_infohash(), name=tdef.get_name_as_unicode())
+                self.torrent_db._db.commit_now()
                 write_my_pref()
             elif self.rtorrent_handler:
                 self.rtorrent_handler.save_torrent(tdef, write_my_pref)
@@ -567,9 +576,6 @@ class TriblerLaunchMany(TaskManager):
         if not hidden:
             self.remove_id(infohash)
 
-        if self.tunnel_community:
-            self.tunnel_community.on_download_removed(d)
-
         return out or succeed(None)
 
     def remove_id(self, infohash):
@@ -902,40 +908,49 @@ class TriblerLaunchMany(TaskManager):
         # Note: session_lock not held
         self.shutdownstarttime = timemod.time()
         if self.credit_mining_manager:
+            self.session.notify_shutdown_state("Shutting down Credit Mining...")
             yield self.credit_mining_manager.shutdown()
         self.credit_mining_manager = None
 
         if self.torrent_checker:
+            self.session.notify_shutdown_state("Shutting down Torrent Checker...")
             yield self.torrent_checker.shutdown()
         self.torrent_checker = None
 
         if self.channel_manager:
+            self.session.notify_shutdown_state("Shutting down Channel Manager...")
             yield self.channel_manager.shutdown()
         self.channel_manager = None
 
         if self.search_manager:
+            self.session.notify_shutdown_state("Shutting down Search Manager...")
             yield self.search_manager.shutdown()
         self.search_manager = None
 
         if self.rtorrent_handler:
+            self.session.notify_shutdown_state("Shutting down Remote Torrent Handler...")
             yield self.rtorrent_handler.shutdown()
         self.rtorrent_handler = None
 
         if self.video_server:
+            self.session.notify_shutdown_state("Shutting down Video Server...")
             yield self.video_server.shutdown_server()
         self.video_server = None
 
         if self.version_check_manager:
+            self.session.notify_shutdown_state("Shutting down Version Checker...")
             self.version_check_manager.stop()
         self.version_check_manager = None
 
         if self.resource_monitor:
+            self.session.notify_shutdown_state("Shutting down Resource Monitor...")
             self.resource_monitor.stop()
         self.resource_monitor = None
 
         self.tracker_manager = None
 
         if self.tftp_handler is not None:
+            self.session.notify_shutdown_state("Shutting down TFTP Handler...")
             yield self.tftp_handler.shutdown()
         self.tftp_handler = None
 
@@ -943,13 +958,16 @@ class TriblerLaunchMany(TaskManager):
             # We unload these overlays manually since the TrustChain has to be unloaded after the tunnel overlay.
             tunnel_community = self.tunnel_community
             self.tunnel_community = None
+            self.session.notify_shutdown_state("Unloading Tunnel Community...")
             yield self.ipv8.unload_overlay(tunnel_community)
             trustchain_community = self.trustchain_community
             self.trustchain_community = None
+            self.session.notify_shutdown_state("Shutting down TrustChain Community...")
             yield self.ipv8.unload_overlay(trustchain_community)
 
         if self.dispersy:
             self._logger.info("lmc: Shutting down Dispersy...")
+            self.session.notify_shutdown_state("Shutting down Dispersy...")
             now = timemod.time()
             try:
                 success = yield self.dispersy.stop()
@@ -964,50 +982,49 @@ class TriblerLaunchMany(TaskManager):
                 self._logger.info("lmc: Dispersy failed to shutdown in %.2f seconds", diff)
 
         if self.ipv8:
+            self.session.notify_shutdown_state("Shutting down IPv8...")
             yield self.ipv8.stop(stop_reactor=False)
 
         if self.metadata_store is not None:
+            self.session.notify_shutdown_state("Shutting down Metadata Store...")
             yield self.metadata_store.close()
         self.metadata_store = None
 
         if self.channelcast_db is not None:
+            self.session.notify_shutdown_state("Shutting down ChannelCast DB...")
             yield self.channelcast_db.close()
         self.channelcast_db = None
 
         if self.votecast_db is not None:
+            self.session.notify_shutdown_state("Shutting down VoteCast DB...")
             yield self.votecast_db.close()
         self.votecast_db = None
 
         if self.mypref_db is not None:
+            self.session.notify_shutdown_state("Shutting down Preference DB...")
             yield self.mypref_db.close()
         self.mypref_db = None
 
         if self.torrent_db is not None:
+            self.session.notify_shutdown_state("Shutting down Torrent DB...")
             yield self.torrent_db.close()
         self.torrent_db = None
 
         if self.peer_db is not None:
+            self.session.notify_shutdown_state("Shutting down Peer DB...")
             yield self.peer_db.close()
         self.peer_db = None
 
-        if self.mainline_dht is not None:
-            from Tribler.Core.DecentralizedTracking import mainlineDHT
-            yield mainlineDHT.deinit(self.mainline_dht)
-        self.mainline_dht = None
-
         if self.torrent_store is not None:
+            self.session.notify_shutdown_state("Shutting down Torrent Store...")
             yield self.torrent_store.close()
         self.torrent_store = None
 
         if self.watch_folder is not None:
+            self.session.notify_shutdown_state("Shutting down Watch Folder...")
             yield self.watch_folder.stop()
         self.watch_folder = None
 
-        # We close the API manager as late as possible during shutdown.
-        if self.api_manager is not None:
-            yield self.api_manager.stop()
-        self.api_manager = None
-
     def network_shutdown(self):
         try:
             self._logger.info("tlm: network_shutdown")
