@@ -6,6 +6,7 @@ from twisted.web.server import NOT_DONE_YET
 
 from Tribler.Core.DownloadConfig import DownloadStartupConfig
 from Tribler.Core.Modules.restapi.util import return_handled_exception
+from Tribler.Core.Utilities.utilities import unichar_string
 from Tribler.Core.simpledefs import DOWNLOAD, UPLOAD, dlstatus_strings, DLMODE_VOD
 import Tribler.Core.Utilities.json_util as json
 
@@ -301,12 +302,12 @@ class DownloadsEndpoint(DownloadBaseEndpoint):
 
         def on_error(error):
             request.setResponseCode(http.INTERNAL_SERVER_ERROR)
-            request.write(json.dumps({"error": error.getErrorMessage()}))
+            request.write(json.dumps({"error": unichar_string(error.getErrorMessage())}))
             request.finish()
 
         uri = parameters['uri'][0]
         if uri.startswith("file:"):
-            download_uri = u"file:%s" % url2pathname(unicode(uri[5:], 'utf-8'))
+            download_uri = u"file:%s" % url2pathname(uri[5:]).decode('utf-8')
         else:
             download_uri = unquote_plus(unicode(uri, 'utf-8'))
         download_deferred = self.session.start_download_from_uri(download_uri, download_config)
