@@ -4,12 +4,14 @@ from binascii import hexlify
 from hashlib import sha256
 import time
 
+from six import string_types
+
 from ...database import database_blob
 from ...keyvault.crypto import default_eccrypto
 from ...messaging.deprecated.encoding import decode, encode
 from ...messaging.serialization import default_serializer
 from .payload import HalfBlockPayload
-from ...util import is_unicode, old_round
+from ...util import old_round
 
 
 GENESIS_HASH = b'0'*32    # ID of the first block of the chain.
@@ -340,7 +342,8 @@ class TrustChainBlock(object):
                 # self counter signs another block (link). If link has a linked block that is not equal to self,
                 # then self is fraudulent, since it tries to countersign a block that is already countersigned
                 linklinked = database.get_linked(link)
-                if linklinked is not None and linklinked.hash != self.hash:
+                if linklinked is not None and linklinked.hash != self.hash and \
+                        link.link_public_key != ANY_COUNTERPARTY_PK:
                     result.err("Double countersign fraud")
 
     def update_chain_consistency(self, prev_blk, next_blk, result):
@@ -404,7 +407,7 @@ class TrustChainBlock(object):
         blk = database.get_latest(public_key)
         ret = cls()
         if link:
-            ret.type = link.type
+            ret.type = link.type if link.link_public_key != ANY_COUNTERPARTY_PK else block_type
             ret.transaction = link.transaction if additional_info is None else additional_info
             ret.link_public_key = link.public_key
             ret.link_sequence_number = link.sequence_number
@@ -442,7 +445,7 @@ class TrustChainBlock(object):
         for key, value in self.__dict__.items():
             if key == 'key' or key == 'serializer' or key == 'crypto' or key == '_transaction':
                 continue
-            if (isinstance(value, str) or is_unicode(value)) and key != "insert_time" and key != "type":
+            if isinstance(value, string_types) and key != "insert_time" and key != "type":
                 yield key, hexlify(value)
             else:
                 yield key, value
