@@ -23,6 +23,7 @@ PAGE_MARKET = 13
 PAGE_MARKET_TRANSACTIONS = 14
 PAGE_MARKET_WALLETS = 15
 PAGE_MARKET_ORDERS = 16
+PAGE_TOKEN_MINING_PAGE = 17
 
 PAGE_CHANNEL_CONTENT = 0
 PAGE_CHANNEL_COMMENTS = 1
@@ -95,3 +96,6 @@ SHUTDOWN_WAITING_PERIOD = 120000
 
 # Garbage collection timer (in minutes)
 GC_TIMEOUT = 10
+
+# Interval for refreshing the results in the debug pane
+DEBUG_PANE_REFRESH_TIMEOUT = 5000   # 5 seconds
